#!/bin/bash
shopt -s extglob

# In the meantime, you've BLASTed the sequences

# This file creates the protein counts and the 16S counts

nSamples=10
WriteDir="Reads";
BLASTDir="BLAST"

Community='Example';

if [[ ! -d $Community ]]; then
    echo "Community $Community does not exist"
    exit
fi

echo  "$Community "

filePath=${Community}/${WriteDir}
outDir=${filePath}/${BLASTDir}

for i in $(seq 1 $nSamples); do
    echo -n -e "Sample $i "

    # This is the file that has all the reads that were BLASTed
    expectedReads=$(zcat ${filePath}/OTUs_${i}-Empirical.fna.filtered.gz | grep '^>' | wc -l);
    echo "($expectedReads reads)";
    
    for blast in 'blastx' 'blastn'; do


	# Here, the BLAST files are broken up into sub-files that are numbered.


    	# Process them away
    	if [ -f ${outDir}/OTUs_${i}-Empirical.fna.1.${blast}-e1.gz ]; then
    	    echo -e "\tFiltering"
	    filterBlastResults.pl -f -e 1 -i 0 -s 0 -a -m 20 -r $expectedReads ${outDir}/OTUs_${i}-Empirical.fna.+([0-9]).${blast}-e1.gz | gzip > ${filePath}/OTUs_${i}-Empirical.fna.${blast}.gz
    	    echo -e "\tGetting highest genes"
	    get_highest_blastKO_or_gene.pl ${filePath}/OTUs_${i}-Empirical.fna.${blast}.gz
    	else
	    # This shouldn't ever happen, but it's nice to have in place."
    	    echo "Sample $i has not been ${blast}'d"
    	fi
	
    done
   
    for blast in 'blastn'; do
     	./correct16SCounts.pl ${filePath}/OTUs_${i}-Empirical.fna.${blast}.highestKOs.counts > ${filePath}/OTUs_${i}-Empirical.fna.${blast}.highestKOs.counts.16S
    done
    
done
