#!/usr/bin/perl
# Original By Elhanan
# Rewritten by Rogan
# Sorts by evalue
use strict;
use Getopt::Long;

my $count = 0;
my $matched = 0;
my $maxEval;
my $minIdentity;
my $minScore;
my $showFiles;
my $shouldRead;
my $evalueField;
my $identityField;
my $scoreField;
my $help;
my $maxMatches;
my $maxMatchesKeepTies;
my $permissive;

GetOptions("maxmatches=i"=>\$maxMatches,"eval=f"=>\$maxEval,"identity=i"=>\$minIdentity,"score=i"=>\$minScore,"file"=>\$showFiles,"reads=i"=>\$shouldRead,"col-evalue=i"=>\$evalueField,"col-identity=i"=>\$identityField,"col-score"=>\$scoreField,"help"=>\$help, "allowties"=>\$maxMatchesKeepTies, "permissive"=>\$permissive);

unless (defined $maxEval) {
  $maxEval = 1e+32;
}
unless (defined $minIdentity) {
  $minIdentity = 0;
}
unless (defined $minScore) {
  $minScore = 0;
}
unless (defined $shouldRead) {
  $shouldRead = -1;
}
unless (defined $identityField) {
  $identityField = 2;
}
unless (defined $evalueField) {
  $evalueField = 3;
}
unless (defined $scoreField) {
  $scoreField = 4;
}
unless (defined $maxMatches) {
  $maxMatches=0;
}
unless (defined $maxMatchesKeepTies) {
  $maxMatchesKeepTies=0;
}
unless (defined $permissive) {
  $permissive=0;
}

if ( defined $help || scalar @ARGV == 0 ) {
  print "$0 -e eval -i identity -s score -r reads -col-identity column -col-evalue column -col-score column \@files...\n";
  exit;
}

print "#eVal $maxEval\n#identity $minIdentity\n#score $minScore\n#max number of matches $maxMatches", $maxMatchesKeepTies? ", keeping ties.":'' , "\n#", join("\n#",@ARGV), "\n";

foreach my $file (@ARGV) {
  # If the file is a gzip, we unzip; otherwise it's untouched
  if ($file =~ /gz$/) {
    $file = "zcat $file |";
  }

  my %Current;

  open (FH, "$file") or die "Can't open $file for reading!\n";
  my $curSeq = '';
  while(<FH>) {	
    chomp;
    # Bounce out uninteresting lines
    next if (/^([\*-]|blast exited|Warning)/ );
    
    # Look for hidden messages
    if (/^#/) {
      $count++ if ( /^# Query/);
      next;
    }

    # Ignore large evals
    my ($seq, $identity, $eval, $score) = (split /\s+/)[0,$identityField,$evalueField,$scoreField];
    
    if ( $permissive ) {
      next unless ( $eval <= $maxEval && $identity >= $minIdentity && $score >= $minScore );
    } else {
      next unless ( $eval < $maxEval && $identity >= $minIdentity && $score >= $minScore );
    }
    
    # New sequence
    if ( $seq ne $curSeq ) {
      # Print out the results
      my $nMatches=0;
      AllEvalues: foreach my $egroup (sort {$a<=>$b} keys %Current) {
	foreach my $result ( @{$Current{$egroup}} ) {
	  print $result, "\n";
	  $nMatches++;
	  # Exit if we have enough and ignore ties
	  if ($maxMatches && $nMatches >= $maxMatches && ! $maxMatchesKeepTies) {
	    last AllEvalues;
	  }
	}
	# We have all the ties
	if ($maxMatches && $nMatches >= $maxMatches ) {
	  last;
	}
      }
      %Current={};
      $matched++;
      $curSeq = $seq;
    }

    push @{$Current{$eval}}, $_;

#    print $_, "\n";
  }
  close(FH);
  
  # Pop out the last
  my $nMatches=0;
  AllEvalues: foreach my $egroup (sort {$a<=>$b} keys %Current) {
    foreach my $result ( @{$Current{$egroup}} ) {
      print $result, "\n";
      $nMatches++;
      # Exit if we have enough and ignore ties
      if ($maxMatches && $nMatches >= $maxMatches && ! $maxMatchesKeepTies) {
	last AllEvalues;
      }
    }
    # We have all the ties
    if ($maxMatches && $nMatches >= $maxMatches ) {
      last;
    }
  }
}

print "# Found $count, blast matched $matched (", $matched/$count, ")\n";
if ($count != $shouldRead) {
  print "# Warning: There should have been $shouldRead reads\n";
  warn "Found $count reads but expected $shouldRead\n";
}
